import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import {
  CacheInterceptor,
  CacheKey,
  Controller,
  UseInterceptors,
  UsePipes,
} from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod } from '@nestjs/microservices';
import { Territory } from '../../domain/entities/territory';
import { FindForPointQuery } from '../../queries/find-for-point.query';
import { TerritoryPresenter } from './territory.presenter';
import { ICollection } from '../../../database/src/interfaces/collection.interface';
import { RpcValidationPipe } from '../../../../utils/pipes/rpc.validation-pipe';
import { FindForPointRequest } from '../../domain/dtos/find-for-point.request';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class TerritoriesController {
  constructor(
    private readonly _queryBus: QueryBus,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  @GrpcMethod('TerritoriesService', 'FindForPoint')
  @UseInterceptors(CacheInterceptor)
  @CacheKey('TerritoriesServiceFindForPoint')
  async findForPoint(
    data: FindForPointRequest,
  ): Promise<ICollection<Territory>> {
    const territoryCollection = await this._queryBus.execute(
      new FindForPointQuery(data),
    );
    return Promise.resolve({
      data: territoryCollection.data.map((territory: Territory) =>
        this._mapper.map(territory, Territory, TerritoryPresenter),
      ),
      total: territoryCollection.total,
    });
  }
}
