import { Test, TestingModule } from '@nestjs/testing';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { FindForPointRequest } from '../../domain/dtos/find-for-point.request';
import { FindForPointUseCase } from '../../domain/usecases/find-for-point.usecase';
import { FindForPointQuery } from '../../queries/find-for-point.query';

const findForPointRequest: FindForPointRequest = new FindForPointRequest();
findForPointRequest.lon = 6.181455;
findForPointRequest.lat = 48.685689;

const findforPointQuery: FindForPointQuery = new FindForPointQuery(
  findForPointRequest,
);

const mockTerritories = [
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    name: 'Nancy',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
    name: 'Meurthe-et-Moselle',
  },
];

const mockTerritoriesRepository = {
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  findForPoint: jest.fn().mockImplementation((query?: FindForPointQuery) => {
    return Promise.resolve(mockTerritories);
  }),
};

describe('FindforPointUseCase', () => {
  let findForPointUseCase: FindForPointUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        FindForPointUseCase,
      ],
    }).compile();

    findForPointUseCase = module.get<FindForPointUseCase>(FindForPointUseCase);
  });

  it('should be defined', () => {
    expect(findForPointUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return an array filled with territories', async () => {
      const territories = await findForPointUseCase.execute(findforPointQuery);

      expect(territories).toBe(mockTerritories);
    });
  });
});
